==> encompass-ecs-0.24.2.tgz: package/encompass-ecs/renderers/entity.ts <==
import { Component } from "../component";
import { DrawComponent } from "../draw_component";
import { EntityChecker } from "../engines/utility/entity_checker";
import { EntityTracker } from "../engines/utility/entity_tracker";
import { Entity } from "../entity";
import { IDrawableEntityTracker } from "../interfaces/drawable_entity_tracker";
import { Renderer } from "../renderer";
import { Type } from "../types/generic_type";

/**
 * An EntityRenderer is an Engine which is responsible for
 * reading Entities with specific Components to draw elements to the screen.
 *
 * EntityRenderers are defined by the Component types they
 * track and by their specific draw_component_type.
 * They also implement a render function.
 *
 * It is an anti-pattern to modify Entities or
 * Components from within a EntityRenderer.
 */
export abstract class EntityRenderer extends Renderer implements IDrawableEntityTracker {
    public abstract readonly component_types: Type<Component>[] = [];

    /**
     * A DrawComponent constructor.
     */
    public abstract draw_component_type: Type<DrawComponent>;

    private entity_tracker = new EntityTracker();

    /**
     * Upon World draw, this function runs for each
     * entity tracked by the Renderer.
     *
     * @param entity A reference to an Entity which
     * is tracked by the Renderer.
     */
    public abstract render(entity: Entity): void;

    public check_and_track_entity(entity_id: number) {
        const entity = this.get_entity(entity_id)!;
        const should_track = this.check_entity(entity);
        if (should_track) { this.entity_tracker.track_entity(entity_id); }
        return should_track;
    }

    public check_and_untrack_entity(entity_id: number) {
        const entity = this.get_entity(entity_id);
        const should_untrack = entity === undefined || !this.check_entity(entity);
        if (should_untrack) {
            this.entity_tracker.untrack_entity(entity_id);
        }
        return should_untrack;
    }

    public is_tracking(id: number) {
        return this.entity_tracker.tracked_ids.has(id);
    }

    protected check_entity(entity: Entity) {
        return EntityChecker.check_entity(entity, this.component_types) &&
               entity.has_component(this.draw_component_type);
    }
}

==> encompass-ecs-0.27.4.tgz: package/encompass-ecs/renderers/entity.ts <==
import { Component } from "../component";
import { DrawComponent } from "../draw_component";
import { EntityChecker } from "../engines/utility/entity_checker";
import { EntityTracker } from "../engines/utility/entity_tracker";
import { Entity } from "../entity";
import { IDrawableEntityTracker } from "../interfaces/drawable_entity_tracker";
import { Renderer } from "../renderer";
import { Type } from "../types/generic_type";

/**
 * An EntityRenderer is an Engine which is responsible for
 * reading Entities with specific Components to draw elements to the screen.
 *
 * EntityRenderers are defined by the Component types they
 * track and by their specific draw_component_type.
 * They also implement a render function.
 *
 * It is an anti-pattern to modify Entities or
 * Components from within a EntityRenderer.
 */
export abstract class EntityRenderer extends Renderer implements IDrawableEntityTracker {
    public readonly component_types: Type<Component>[] = [];

    /**
     * A DrawComponent constructor.
     */
    public readonly draw_component_type: Type<DrawComponent>;

    private entity_tracker = new EntityTracker();

    /**
     * Upon World draw, this function runs for each
     * entity tracked by the Renderer.
     *
     * @param entity A reference to an Entity which
     * is tracked by the Renderer.
     */
    public abstract render(entity: Entity): void;

    public check_and_track_entity(entity_id: number) {
        const entity = this.get_entity(entity_id)!;
        const should_track = this.check_entity(entity);
        if (should_track) { this.entity_tracker.track_entity(entity_id); }
        return should_track;
    }

    public check_and_untrack_entity(entity_id: number) {
        const entity = this.get_entity(entity_id);
        const should_untrack = entity === undefined || !this.check_entity(entity);
        if (should_untrack) {
            this.entity_tracker.untrack_entity(entity_id);
        }
        return should_untrack;
    }

    public is_tracking(id: number) {
        return this.entity_tracker.tracked_ids.has(id);
    }

    protected check_entity(entity: Entity) {
        return EntityChecker.check_entity(entity, this.component_types) &&
               entity.has_component(this.draw_component_type);
    }
}
